<?php

declare(strict_types=1);

namespace apexl\Io\modules\file\enums\permissions;

use apexl\Io\modules\user\traits\IsPermissionTrait;
use apexl\Io\modules\user\interfaces\PermissionInterface;

enum File implements PermissionInterface, \JsonSerializable
{
    use IsPermissionTrait;

    case UPLOAD_FILES;
}
